==> easy_entity_field-1.0.x-dev.tar.gz: easy_entity_field/src/Form/EasyEntityFieldAddForm.php <==
<?php

namespace Drupal\easy_entity_field\Form;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for the "field storage" add page.
 *
 * @internal
 */
class EasyEntityFieldAddForm extends FormBase {

  /**
   * The name of the entity type.
   *
   * @var string
   */
  protected $entityTypeId;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypePluginManager;

  /**
   * Constructs a new FieldStorageAddForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_plugin_manager
   *   The field type plugin manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface|null $entity_field_manager
   *   (optional) The entity field manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, FieldTypePluginManagerInterface $field_type_plugin_manager, EntityFieldManagerInterface $entity_field_manager = NULL) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fieldTypePluginManager = $field_type_plugin_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'field_ui_field_storage_add_form';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $entity_type_id = NULL) {

    if (!$form_state->get('entity_type_id')) {
      $form_state->set('entity_type_id', $entity_type_id);
    }

    $this->entityTypeId = $form_state->get('entity_type_id');
    $field_type_options = \Drupal::service('plugin.manager.easy_entity_field')->getDefaultFields();
    $form['add'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['form--inline', 'clearfix']],
    ];

    $form['add']['field_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Add a new field'),
      '#options' => $field_type_options,
      '#empty_option' => $this->t('- Select a field type -'),
    ];

    // Field label and field_name.
    $form['new_field_wrapper'] = [
      '#type' => 'container',
      '#states' => [
        '!visible' => [
          ':input[name="field_type"]' => ['value' => ''],
        ],
      ],
    ];
    $form['new_field_wrapper']['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#size' => 15,
    ];

    $form['new_field_wrapper']['field_name'] = [
      '#type' => 'machine_name',
      '#size' => 15,
      '#description' => $this->t('A unique machine-readable name containing letters, numbers, and underscores.'),
      // Calculate characters depending on the length of the field prefix
      // setting. Maximum length is 32.
      '#machine_name' => [
        'source' => ['new_field_wrapper', 'label'],
        'exists' => [$this, 'fieldNameExists'],
      ],
      '#required' => FALSE,
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save and continue'),
      '#button_type' => 'primary',
    ];

    $form['#attached']['library'][] = 'field_ui/drupal.field_ui';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    // Validate if any information was provided in the 'add new field' case.
    if ($form_state->getValue('field_type')) {
      // Missing label.
      if (!$form_state->getValue('label')) {
        $form_state->setErrorByName('label', $this->t('Add new field: you need to provide a label.'));
      }

      // Missing field name.
      if (!$form_state->getValue('field_name')) {
        $form_state->setErrorByName('field_name', $this->t('Add new field: you need to provide a machine name for the field.'));
      }
    }
  }


  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $values = $form_state->getValues();
    $entity_type = $this->entityTypeManager->getDefinition($this->entityTypeId);
    // Create new field.
    if ($values['field_type']) {
      $field_values = [
        'label' => $values['label'],
        'field_name' => $values['field_name'],
        'entity_type' => $this->entityTypeId,
        'field_type' => $values['field_type'],
        'translatable' => $entity_type->isTranslatable(),
      ];

      $widget_id = $formatter_id = NULL;
      $widget_settings = $formatter_settings = [];
      $storage_settings = $field_settings = [];
      // Check if we're dealing with a preconfigured field.
      if (strpos($field_values['field_type'], 'field_ui:') !== FALSE) {
        [, $field_type, $option_key] = explode(':', $field_values['field_type'], 3);
        $field_values['field_type'] = $field_type;

        $field_definition = $this->fieldTypePluginManager->getDefinition($field_type);
        $options = $this->fieldTypePluginManager->getPreconfiguredOptions($field_definition['id']);
        $field_options = $options[$option_key];

        // Merge in preconfigured field storage options.
        if (isset($field_options['field_storage_config'])) {
          $storage_settings = $field_options['field_storage_config']['settings'];
        }

        // Merge in preconfigured field options.
        if (isset($field_options['field_config'])) {
          $field_values['required'] = $field_options['field_config']['required'];
          $field_settings = $field_options['field_config']['settings'];
        }

        $widget_id = $field_options['entity_form_display']['type'] ?? NULL;
        $widget_settings = $field_options['entity_form_display']['settings'] ?? [];
        $formatter_id = $field_options['entity_view_display']['type'] ?? NULL;
        $formatter_settings = $field_options['entity_view_display']['settings'] ?? [];
      }
      else {
        if (!empty($this->fieldTypePluginManager->getDefaultStorageSettings($values['field_type']))) {
          $storage_settings = $this->fieldTypePluginManager->getDefaultStorageSettings($values['field_type']);
        }

        if (!empty($this->fieldTypePluginManager->getDefaultFieldSettings($values['field_type']))) {
          $field_settings = $this->fieldTypePluginManager->getDefaultFieldSettings($values['field_type']);
        }
      }

      $field_values['display']['formatter_type'] = $formatter_id;
      $field_values['display']['formatter_settings'] = $formatter_settings;
      $field_values['display']['widget_type'] = $widget_id;
      $field_values['display']['widget_settings'] = $widget_settings;

      $field_values['field_settings'] = $field_settings;
      $field_values['storage_settings'] = $storage_settings;
      // Create the field storage and field.
      try {
        $field = $this->entityTypeManager->getStorage('easy_entity_field')->create($field_values);
        $field->save();

        $form_state->setRedirectUrl($field->toUrl('storage-form'));
      } catch (\Exception $e) {
        $this->messenger()->addError($this->t('There was a problem creating field %label: @message', ['%label' => $values['label'], '@message' => $e->getMessage()]));
      }
    }
  }

  public function fieldNameExists($value, $element, FormStateInterface $form_state) {

    $field_storage_definitions = $this->entityFieldManager->getFieldStorageDefinitions($this->entityTypeId);
    return isset($field_storage_definitions[$value]);
  }

}

==> easy_entity_field-2.0.x-dev.tar.gz: easy_entity_field/src/Form/EasyEntityFieldAddForm.php <==
<?php

namespace Drupal\easy_entity_field\Form;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for the "field storage" add page.
 *
 * @internal
 */
class EasyEntityFieldAddForm extends FormBase {

  /**
   * The name of the entity type.
   *
   * @var string
   */
  protected $entityTypeId;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypePluginManager;

  /**
   * Constructs a new FieldStorageAddForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_plugin_manager
   *   The field type plugin manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface|null $entity_field_manager
   *   (optional) The entity field manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, FieldTypePluginManagerInterface $field_type_plugin_manager, EntityFieldManagerInterface $entity_field_manager = NULL) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fieldTypePluginManager = $field_type_plugin_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'field_ui_field_storage_add_form';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $entity_type_id = NULL) {
    if (!$form_state->get('entity_type_id')) {
      $form_state->set('entity_type_id', $entity_type_id);
    }

    $this->entityTypeId = $form_state->get('entity_type_id');

    $form['add'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['form--inline', 'clearfix']],
    ];

    $field_type_options = \Drupal::service('plugin.manager.easy_entity_field')->getDefaultFields();
    $form['add']['field_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Add a new field'),
      '#options' => $field_type_options,
      '#empty_option' => $this->t('- Select a field type -'),
    ];

    // Field label and field_name.
    $form['new_field_wrapper'] = [
      '#type' => 'container',
      '#states' => [
        '!visible' => [
          ':input[name="field_type"]' => ['value' => ''],
        ],
      ],
    ];

    $form['new_field_wrapper']['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#size' => 15,
    ];

    $form['new_field_wrapper']['field_name'] = [
      '#type' => 'machine_name',
      '#size' => 15,
      '#description' => $this->t('A unique machine-readable name containing letters, numbers, and underscores.'),
      // Calculate characters depending on the length of the field prefix
      // setting. Maximum length is 32.
      '#machine_name' => [
        'source' => ['new_field_wrapper', 'label'],
        'exists' => [$this, 'fieldNameExists'],
      ],
      '#required' => FALSE,
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save and continue'),
      '#button_type' => 'primary',
    ];

    //    $form['#attached']['library'][] = 'field_ui/drupal.field_ui';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate if any information was provided in the 'add new field' case.
    if ($form_state->getValue('field_type')) {
      // Missing label.
      if (!$form_state->getValue('label')) {
        $form_state->setErrorByName('label', $this->t('Add new field: you need to provide a label.'));
      }

      // Missing field name.
      if (!$form_state->getValue('field_name')) {
        $form_state->setErrorByName('field_name', $this->t('Add new field: you need to provide a machine name for the field.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $entity_type = $this->entityTypeManager->getDefinition($this->entityTypeId);
    // Create new field.
    if ($values['field_type']) {
      $field_values = [
        'label' => $values['label'],
        'field_name' => $values['field_name'],
        'entity_type' => $this->entityTypeId,
        'field_type' => $values['field_type'],
        'translatable' => $entity_type->isTranslatable(),
      ];

      $widget_id = $formatter_id = NULL;
      $widget_settings = $formatter_settings = [];
      $storage_settings = $field_settings = [];
      // Check if we're dealing with a preconfigured field.
      if (str_contains($field_values['field_type'], 'field_ui:')) {
        [, $field_type, $option_key] = explode(':', $field_values['field_type'], 3);
        $field_values['field_type'] = $field_type;

        $field_definition = $this->fieldTypePluginManager->getDefinition($field_type);
        $options = $this->fieldTypePluginManager->getPreconfiguredOptions($field_definition['id']);
        $field_options = $options[$option_key];

        // Merge in preconfigured field storage options.
        if (isset($field_options['field_storage_config'])) {
          $storage_settings = $field_options['field_storage_config']['settings'];
        }

        // Merge in preconfigured field options.
        if (isset($field_options['field_config'])) {
          $field_values['required'] = $field_options['field_config']['required'];
          $field_settings = $field_options['field_config']['settings'];
        }

        $widget_id = $field_options['entity_form_display']['type'] ?? NULL;
        $widget_settings = $field_options['entity_form_display']['settings'] ?? [];
        $formatter_id = $field_options['entity_view_display']['type'] ?? NULL;
        $formatter_settings = $field_options['entity_view_display']['settings'] ?? [];
      }
      else {
        if (!empty($this->fieldTypePluginManager->getDefaultStorageSettings($values['field_type']))) {
          $storage_settings = $this->fieldTypePluginManager->getDefaultStorageSettings($values['field_type']);
        }

        if (!empty($this->fieldTypePluginManager->getDefaultFieldSettings($values['field_type']))) {
          $field_settings = $this->fieldTypePluginManager->getDefaultFieldSettings($values['field_type']);
        }
      }

      $field_values['display']['formatter_type'] = $formatter_id;
      $field_values['display']['formatter_settings'] = $formatter_settings;
      $field_values['display']['widget_type'] = $widget_id;
      $field_values['display']['widget_settings'] = $widget_settings;

      $field_values['field_settings'] = $field_settings;
      $field_values['storage_settings'] = $storage_settings;
      // Create the field storage and field.
      try {
        $field = $this->entityTypeManager->getStorage('easy_entity_field')->create($field_values);
        $field->save();

        $form_state->setRedirectUrl($field->toUrl('storage-form'));
      }
      catch (\Exception $e) {
        $this->messenger()->addError($this->t('There was a problem creating field %label: @message', ['%label' => $values['label'], '@message' => $e->getMessage()]));
      }
    }
  }

  /**
   * Checks if a field machine name is taken.
   *
   * @param string $value
   *   The machine name, not prefixed.
   * @param array $element
   *   An array containing the structure of the 'field_name' element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return bool
   *   Whether or not the field machine name is taken.
   */
  public function fieldNameExists($value, $element, FormStateInterface $form_state) {
    $field_storage_definitions = $this->entityFieldManager->getFieldStorageDefinitions($this->entityTypeId);
    return isset($field_storage_definitions[$value]);
  }

}
